:
# RCSid:
#	$Id: sb-funcs.sh,v 1.43 2021/10/22 05:32:31 sjg Exp $
#
#	@(#) Copyright (c) 2009-2020 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

Myname=${Myname:-`basename $0 .sh`}
Mydir=${Mydir:-`dirname $0`}
pwd=${pwd:-'pwd'}
case "$Mydir" in
.) Mydir=`$pwd`;;
esac
SB_TOOLS=${SB_TOOLS:-$Mydir}
$_OS_SH . $SB_TOOLS/os.sh
$_HOOKS_SH . $SB_TOOLS/hooks.sh
$_ADD_PATH_SH . $SB_TOOLS/add_path.sh
$_TEST_OPTS_SH . $SB_TOOLS/test_opt.sh

ev=.sandbox-env
rc=.sandboxrc

# set isPOSIX_SHELL={true,false}
# so we can use 'if $isPOSIX_SHELL; then'
if (echo ${PATH%:*}) > /dev/null 2>&1; then
    isPOSIX_SHELL=true
    # you need to eval $local var
    local=local
    # reduce the cost of these
    basename() {
        local b=${1%$2}
        echo ${b##*/}
    }
    dirname() {
        case "$1" in
        */*) echo ${1%/*};;
        *) echo .;;
        esac
    }
    true() {
        return 0
    }
else
    isPOSIX_SHELL=false
    local=:
    false() {
        return 1
    }
fi
    
# turn something into a useful var name
varName() {
    case "$1" in
    *[!A-Za-z0-9_]*) vn=`echo $1 | sed 's,[^A-Za-z0-9_],_,g'`;;
    *) vn=$1;;
    esac
    echo $vn
}

evalVar() {
    val="$@"
    while :
    do
        case "$val" in
	*\$*) eval "val=\"$val\"";;
	*) break;;
	esac
    done
    echo "$val"
}
    
# we sometimes need these as var names
# MYNAME is our canonical name
# Myname is from $0
for n in Myname MYNAME
do
    eval vn=\$$n
    vn=`varName $vn`
    eval var${n}=$vn
done
# compute this once
if test x$varMyname = x$varMYNAME; then
    SB_VARMYNAME_LIST=$varMyname
else
    SB_VARMYNAME_LIST="$varMyname $varMYNAME"
fi

# find_it [--start $dir][-$test][--{path,dir}] $path ...
# find the first $path in $start or above
# if --path report $dir/$path otherwise just $dir where $path found
find_it() {
    _t=-s
    _start=.
    _what=dir
    while :
    do
	case "$1" in
	-?) _t=$1; shift;;
	--path) _what=path; shift;;
	--dir) _what=dir; shift;;
	--start) _start=$2; shift; shift;;
	*) break;;
	esac
    done
    # try to make sure we start in the right place
    # symlinks can wreak havoc
    case `type realpath 2>&1` in
    */*) _start=`realpath $_start`;;
    esac
    'cd' "$_start" > /dev/null 2>&1 || return 1
    while :
    do
	here=`$pwd`
	for it in "$@"
	do
	    if test $_t $it; then
		case "$_what" in
		path) echo $here/$it;;
		*) echo $here;;
		esac
		return
	    fi
	done
	case $here in
	/) return;;
	esac
	'cd' "$here/.."
    done
}

find_sb() {
    find_it --start ${1:-.} --dir $ev
    
}

error() {
    echo "ERROR: $@" >&2
    exit 1
}

# requires 'local' for source_file etc to have correct values when
# used recursively.
# --once avoids repeats
# --one  stops after first one we find
sb_included=
source_rc() {
    eval $local f source_dir source_file _0 _1

    _0=: _1=:
    while :
    do
        case "$1" in
        --once) _0=; shift;;
        --one) _1=; shift;;
        *) break;;
        esac
    done
    for f in "$@"
    do
        [ -s $f ] || continue
        case $f in
        */*) source_dir=`dirname $f` source_file=`basename $f`;;
        *) source_dir=. source_file=$f;;
        esac
        source_dir=`'cd' "$source_dir" && 'pwd'`
        : is $_0$source_dir/$source_file in ,$sb_included,
        case ",$sb_included," in
        *,$_0$source_dir/$source_file,*) continue;;
        esac
        sb_included=$sb_included,$source_dir/$source_file
        . $f
        $_1 return $?
    done
}

##
# sb_run_hooks name [args]
# we run sb_${name}_hooks ${varMyname}_${name}_hooks and
# ${varMYNAME}_${name}_hooks passing args provided.
#
sb_run_hooks() {
    _n=$1; shift

    case "$_n" in
    run) _p1=;;
    *) _p1=sb;;
    esac
    for _p in $_p1 $SB_VARMYNAME_LIST
    do
        run_hooks ${_p}_${_n}_hooks "$@"
    done
}

sb_hooks() {
    'cd' "$1" || exit 1
    SB=`$pwd`
    SB_NAME=`basename $SB`
    export SB SB_NAME

    source_rc --once ../$rc ./$rc
    sb_run_hooks init
    source_rc ./$ev
    sb_run_hooks setup
}

add_list() {
    _list=$1; shift
    eval "$_list=\"\$$_list $@\""
}

sort_list() {
    case "$1" in
    --) _u=; shift;;
    -[run]*) _u=$1; shift;;
    *) _u=;;
    esac
    case "$1" in
    "") return;;
    esac
    for i in "$@"
    do
        echo $i
    done | sort $_u
}

# we want to re-use this
extract_doc() {
    case "$1" in
    eod=*) eval "$1"; shift;;
    *) eod=${END_HELP_MARKER:-RCSid};;
    esac
    for f in "$@"
    do
        sed -n -e "1d;/$eod/,\$d" -e '/^#\.[a-z]/d' -e '/^#/s,^# *,,p' $f
    done
}
        
# an easy way to add --help support to shell scripts
# even those that use getopt(1) is (before that) do:
# # Yes I mean $* below, and the spaces.
# case " $* " in *" --help "*) __help;; esac
_doc_() {
    extract_doc ${1:-$0}
    run_hooks doc_hooks
    exit 0
}

_help_() {
    case "$1" in
    ""|help) topic=;;
    _*) topic=$1;;
    *) topic=_$1;;
    esac

    for n in $SB_VARMYNAME_LIST
    do
	# look for hooks
	for hl in ${n}_help${topic}_hooks ${n}${topic:-_help}_hooks help${topic}_hooks
	do
	    eval hh=\$$hl
	    if [ -n "$hh" ]; then
		run_hooks $hl
		exit 0
	    fi
	done
	# look for function
	for hf in ${n}_help${topic} help${topic}
	do
	    # type is available in most shell's post 1980
	    # users who alias ${n}_help to something
	    # that does not exist, deserve to lose.
	    : hf=$hf
	    case `(type $hf) 2>&1` in
	    *function*)
		$hf
		exit 0
		;;
	    esac
	done
    done
    case "$topic" in
    "") __doc;;
    *) error no help$topic found;;
    esac
    exit 0
}


__doc() {
    _doc_ "$@" | ${PAGER:-more}
    exit 0
}

__help() {
    _help_ "$@" | ${PAGER:-more}
    exit 0
}
